using System;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using System.Threading;
using Microsoft.Win32;

namespace LanmarkControls.LECUtils {

/*****************************************************************************/
public sealed class Utils
{
   /////////////////////////////////////////////////////////////////////////
   
   public static void WriteRegistry()
   {
      // creo le chiavi di registro HKEY_LOCAL_MACHINE\SOFTWARE\Alase Technologies\WinLaseServer\LEC
      RegistryKey localMachine = Registry.LocalMachine;
      RegistryKey software = localMachine.CreateSubKey("SOFTWARE");
      RegistryKey alaseTechnologies = software.CreateSubKey("Alase Technologies");
      RegistryKey winLaseServer = alaseTechnologies.CreateSubKey("WinLaseServer");
      RegistryKey lec = winLaseServer.CreateSubKey("LEC");
      // aggiungo i valori alle chiavi
      lec.SetValue("ClockTick", 0x0000000F);
      lec.SetValue("MaxTransmitSize", 0x0003D090);
      lec.SetValue("Keepalive", 0x0000000A);
      lec.SetValue("TransmitPeriod", 0x00000005);
      lec.SetValue("TransmitTimeout", 0x000000C8);
      
      localMachine.Close();         
   }

	/////////////////////////////////////////////////////////////////////////
	
	public static void ReadFromXML(string pstrData,  int iElementSkipCount, string strSearchElement, string strSearchAttribute, out string strOutString)
	{
		strOutString = "";

		if (pstrData.Length == 0)
			return;

		StringReader strRead = new StringReader(pstrData);
		XmlTextReader xmlRead = new XmlTextReader(strRead);

		ReadFromXML(xmlRead, iElementSkipCount, strSearchElement, strSearchAttribute, out strOutString);

		xmlRead.Close();
		strRead.Close();
	}
	
	/////////////////////////////////////////////////////////////////////////

	private static void ReadFromXML(StreamReader poData,  int iElementSkipCount, string strSearchElement, string strSearchAttribute, out string strOutString)
	{
		strOutString = "";

		XmlTextReader xmlRead = new XmlTextReader(poData);

		ReadFromXML(xmlRead, iElementSkipCount, strSearchElement, strSearchAttribute, out strOutString);

		xmlRead.Close();
	}

	/////////////////////////////////////////////////////////////////////////
	// Reads either an Element Text Value or an Element's Attribute.
	// If Attribute search param is set to null then we are looking 
	// for Element Text Value
	//
	
	private static void ReadFromXML(XmlTextReader pxmlRead, int iElementSkipCount, string strSearchElement, string strSearchAttribute, out string strOutString)
	{
		bool fContinueSearch = true;
		bool fElementFound = false;
		int	iCnt = 0;

		strOutString = "";

		while (fContinueSearch)
		{
			switch (pxmlRead.NodeType) 
			{
				case XmlNodeType.Element:
					if (pxmlRead.Name == strSearchElement)
					{
						if (iCnt++ < iElementSkipCount) break;

						fElementFound = true;

//						Console.Write("Element<{0}> ", pxmlRead.Name);
						if (null != strSearchAttribute)
						{
							if ((0 != pxmlRead.AttributeCount) && (pxmlRead.MoveToAttribute(strSearchAttribute)))
							{
								fContinueSearch = false;

								while (pxmlRead.ReadAttributeValue())
								{
									strOutString = pxmlRead.Value;
//									Console.Write("Attrib<{0}>= '{1}'", strSearchAttribute, strOutString);
								} 
							}
							else
							{
//								Console.Write("Error: No Attrib<{0}> ", strSearchAttribute);
							}
						}
                        if (pxmlRead.IsEmptyElement)
                            fContinueSearch = false;
                    }
                    break;
				case XmlNodeType.Text:
					if (fElementFound)
					{
						fContinueSearch = false;

						strOutString = pxmlRead.Value;
						//Console.Write("Text: <{0}>\n", strOutString);
					}
					break;
				case XmlNodeType.CDATA:
					//					Console.Write("CDATA: <![CDATA[{0}]]>\n", pxmlRead.Value);
					break;
				case XmlNodeType.ProcessingInstruction:
					//					Console.Write("<?{0} {1}?>\n", pxmlRead.Name, pxmlRead.Value);
					break;
				case XmlNodeType.Comment:
					//					Console.Write("<!--{0}-->", pxmlRead.Value);
					break;
				case XmlNodeType.XmlDeclaration:
					//					Console.Write("Comment: <?xml version='1.0'?>\n");
					break;
				case XmlNodeType.Document:
					//					Console.Write("Document: \n");
					break;
				case XmlNodeType.DocumentType:
					//					Console.Write("DocumentType: <!DOCTYPE {0} [{1}]\n", pxmlRead.Name, pxmlRead.Value);
					break;
				case XmlNodeType.EntityReference:
					//					Console.Write("EntityReference: <{0}>\n", pxmlRead.Name);
					break;
				case XmlNodeType.EndElement:
					//					Console.Write("EndElement: </{0}>\n", pxmlRead.Name);
                    if (pxmlRead.Name == strSearchElement)
                        fContinueSearch = false;
                    break;
				default :
					//					Console.Write("default:  Name = '{0}' Value = '{1}'", pxmlRead.Name, pxmlRead.Value);
					break;
			}       
			if (!pxmlRead.Read())
				break;
		} // end of While Loop

	}

	/////////////////////////////////////////////////////////////////////////

	public static void WriteToXML(StreamReader poData,  int iElementSkipCount, string strSearchParent, string strSearchElement, string strSearchAttribute, string strInString)
	{
		// Create the XmlDocument.
		XmlDocument xmlDoc = new XmlDocument();
		xmlDoc.Load(poData);

		WriteToXML(xmlDoc, iElementSkipCount, strSearchParent, strSearchElement, strSearchAttribute, strInString);
	}

	/////////////////////////////////////////////////////////////////////////
	public static void WriteToXML(ref string pstrData,  int iElementSkipCount, string strSearchParent, string strSearchElement, string strSearchAttribute, string strInString)
	{
		if (pstrData.Length == 0)
			return;

		// Create the XmlDocument.
		XmlDocument xmlDoc = new XmlDocument();
		xmlDoc.Load(new StringReader(pstrData));

		WriteToXML(xmlDoc, iElementSkipCount, strSearchParent, strSearchElement, strSearchAttribute, strInString);

		TextWriter stringWriter = new StringWriter();
		xmlDoc.Save(stringWriter);
		pstrData = stringWriter.ToString();
//		Console.Write(stringWriter.ToString());

//		xmlDoc.Close();
	}

	/////////////////////////////////////////////////////////////////////////
	// Writes either an Element Text Value or an Element's Attribute.
	// If Attribute search param is set to null then we are looking 
	// for Element Text Value
	//
	public static void WriteToXML(XmlDocument xmlDoc, int iElementSkipCount, string strSearchParent, string strSearchElement, string strSearchAttribute, string strInString)
	{
		XmlNodeList elementList = xmlDoc.GetElementsByTagName(strSearchElement);

        if (0 == elementList.Count) return;

		// Search for matching parent
        while ("" != strSearchParent  &&  elementList.Item(iElementSkipCount).ParentNode.Name != strSearchParent)
		{
			string str = elementList.Item(iElementSkipCount).ParentNode.Name;
			iElementSkipCount++;
            if (iElementSkipCount >= elementList.Count) return;
        }

		if (null == strSearchAttribute)
		{
			if (null != elementList)
				// write element value
				elementList.Item(iElementSkipCount).InnerText = strInString;
			//else
         //       Console.Write("Has No Element<" + strSearchElement.ToString()+ "> ");
		}
		else	// we are searching for an Attribute
		{
			if ((null != elementList.Item(iElementSkipCount)) && 
			    (null != elementList.Item(iElementSkipCount).Attributes) && 
				(null != elementList.Item(iElementSkipCount).Attributes[strSearchAttribute]) )
			{
				// write Attribute value
				elementList.Item(iElementSkipCount).Attributes[strSearchAttribute].Value = strInString;
			}
			//else
			//	Console.Write("Has No Attrib<" + strSearchAttribute.ToString() + "> ");
		}

	}

	/////////////////////////////////////////////////////////////////////////
	static int readXmlFile2(ref string stream, out string strName)
		//	static int readXmlFile2( )
		//		static void Main(string[] args)
	{
		strName = "JoseOffice (00:50:c2:4f:a0:07)";

		String sContactID, sDuplicateID;
		String sXml =
			@"<BusinessEntity xsi:type='DynamicEntity' Name='contact'
				xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
				xmlns='http://schemas.microsoft.com/crm/2006/WebServices'>
					<Properties>
						<Property xsi:type='KeyProperty' Name='contactid'>
							<Value>{80FB716B-B189-DA11-84BD-0003FFC6732A}</Value>
						</Property>
						<Property xsi:type='StringProperty' Name='duplicateid'>
							<Value>{8e0c0022-2078-4017-9630-6453e80695c0}</Value>
						</Property>
					</Properties>
				</BusinessEntity>";

		XmlDocument xmlDoc = new XmlDocument();
		XmlNamespaceManager xmlNSMgr = new XmlNamespaceManager(xmlDoc.NameTable);

		try
		{
			xmlDoc.LoadXml(sXml);

			// define default NS using prefix "def"
			xmlNSMgr.AddNamespace("def", "http://schemas.microsoft.com/crm/2006/WebServices");

			//Console.Write("Here 1");
			// get contactid
			sContactID = xmlDoc.SelectSingleNode("//def:Property[@Name='contactid']/def:Value", xmlNSMgr).InnerText;
			//Console.Write("Here 2");

			// get duplicateid
			sDuplicateID = xmlDoc.SelectSingleNode("//def:Property[@Name='duplicateid']/def:Value", xmlNSMgr).InnerText;

			//Console.Write("Contact ID: " + sContactID);
			//Console.Write("Duplicate ID: " + sDuplicateID);
		}
		catch (XmlException e)
		{
			//Console.Write("Error loading XML: " + e.Message);
		}
		catch (Exception e)
		{
			//Console.Write("Error: " + e.Message);
		}

		//Console.Write("\nPress any key to continue: ");
		//		Console.ReadKey(true);
		return 0;
	}
}

}